﻿'TBDateEdit Server Control for ASP.NET 2.0
'原創: ASP.NET 魔法學院 http://blog.blueshop.com.tw/jeff377/
'備註: 
'TBDateEdit 控制項的 JavaScript 腳本是使用 The Coolest DHTML Calendar
'The Coolest DHTML Calendar 相關授權聲明及最新程式請參閱 http://www.dynarch.com/projects/calendar/

Imports System
Imports System.Collections.Generic
Imports System.Collections.Specialized
Imports System.ComponentModel
Imports System.Text
Imports System.Drawing
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls

Namespace WebControls
    ''' <summary>
    ''' 日期控制項。
    ''' </summary>
    < _
    Description("日期控制項"), _
    DefaultProperty("Text"), _
    DefaultEvent("TextChanged"), _
    Designer(GetType(Bee.Web.WebControls.Design.TBDateEditDesigner)), _
    ToolboxBitmap(GetType(TBToolboxBitmap), "TBDateEdit.bmp"), _
    ToolboxData("<{0}:TBDateEdit runat=server></{0}:TBDateEdit>") _
    > _
    Public Class TBDateEdit
        Inherits System.Web.UI.WebControls.CompositeControl

        ''' <summary>
        ''' 日期選單風格樣式。
        ''' </summary>
        Public Enum ECalendarStyle
            ''' <summary>
            ''' Blue 風格樣式。
            ''' </summary>
            Blue = 0
            ''' <summary>
            ''' Winter 風格樣式。
            ''' </summary>
            Winter = 1
            ''' <summary>
            ''' Summer 風格樣式。
            ''' </summary>
            Summer = 2
            ''' <summary>
            ''' Green 風格樣式。
            ''' </summary>
            Green = 3
            ''' <summary>
            ''' System 風格樣式。
            ''' </summary>
            System = 4
            ''' <summary>
            ''' Tas 風格樣式。
            ''' </summary>
            Tas = 5
            ''' <summary>
            ''' Win2k_1 風格樣式。
            ''' </summary>
            Win2k_1 = 6
            ''' <summary>
            ''' Win2k_2 風格樣式。
            ''' </summary>
            Win2k_2 = 7
            ''' <summary>
            ''' Win2k_cold_1 風格樣式。
            ''' </summary>
            Win2k_cold_1 = 8
            ''' <summary>
            ''' Win2k_cold_2 風格樣式。
            ''' </summary>
            Win2k_cold_2 = 9
        End Enum

        ''' <summary>
        ''' 日期選單語系。
        ''' </summary>
        Public Enum ELang
            ''' <summary>
            ''' 繁體中文。
            ''' </summary>
            Big5
            ''' <summary>
            ''' 簡體中文。
            ''' </summary>
            Gb
            ''' <summary>
            ''' 英文。
            ''' </summary>
            Eng
        End Enum

        Private FTextBox As New TextBox()
        Private FButton As New HtmlControls.HtmlButton()
        Private FReadOnly As Boolean = False
        Private FScriptPath As String = "~/BeeLibClient/jscalendar/"
        Private FFirstDayOfWeek As FirstDayOfWeek = UI.WebControls.FirstDayOfWeek.Default
        Private FShowWeekNumbers As Boolean = False
        Private FCalendarStyle As ECalendarStyle = ECalendarStyle.Blue
        Private FLang As ELang = ELang.Big5
        Private FShowTime As Boolean = False

        ''' <summary>
        ''' 文字內容改變時所引發的事件。
        ''' </summary>
        Public Event TextChanged(ByVal sender As Object, ByVal e As System.EventArgs)

        Public Sub New()
            AddHandler FTextBox.TextChanged, AddressOf TextBox_TextChanged
        End Sub

        ''' <summary>
        ''' 攔截 TextBox 的 TextChanged 事件。
        ''' </summary>
        Private Sub TextBox_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs)
            '引發 TextChanged 事件
            RaiseEvent TextChanged(Me, e)
        End Sub

        ''' <summary>
        ''' 用戶端指令碼路徑。
        ''' </summary>
        < _
        Description("用戶端指令碼路徑。"), _
        Category("Behavior"), _
        DefaultValue("~/BeeLibClient/jscalendar/") _
        > _
        Public Property ScriptPath() As String
            Get
                Return FScriptPath
            End Get
            Set(ByVal value As String)
                FScriptPath = value
            End Set
        End Property

        ''' <summary>
        ''' 是否觸發 PostBack 動作。
        ''' </summary>
        < _
        Description("是否觸發 PostBack 動作。"), _
        DefaultValue(False) _
        > _
        Public Property AutoPostBack() As Boolean
            Get
                Return FTextBox.AutoPostBack
            End Get
            Set(ByVal value As Boolean)
                FTextBox.AutoPostBack = value
            End Set
        End Property

        ''' <summary>
        ''' 輸入文字內容。
        ''' </summary>
        < _
        Description("輸入文字內容。"), _
        Category("Appearance"), _
        Bindable(True), _
        DefaultValue("") _
        > _
        Public Property Text() As String
            Get
                Return FTextBox.Text
            End Get
            Set(ByVal value As String)
                FTextBox.Text = value
            End Set
        End Property

        ''' <summary>
        ''' 文字框是否唯讀，只允許使用日期選單。
        ''' </summary>
        < _
        Description("文字框是否唯讀，只允許使用日期選單。"), _
        Bindable(True), _
        Themeable(False), _
        Category("Behavior"), _
        DefaultValue(False) _
        > _
        Public Property [ReadOnly]() As Boolean
            Get
                Return FReadOnly
            End Get
            Set(ByVal value As Boolean)
                FReadOnly = value
            End Set
        End Property

        ''' <summary>
        ''' 一週的第一天。
        ''' </summary>
        < _
        Description("一週的第一天。"), _
        Category("Appearance"), _
        DefaultValue(GetType(FirstDayOfWeek), "Default") _
        > _
        Public Property FirstDayOfWeek() As FirstDayOfWeek
            Get
                Return FFirstDayOfWeek
            End Get
            Set(ByVal value As FirstDayOfWeek)
                FFirstDayOfWeek = value
            End Set
        End Property

        ''' <summary>
        ''' 顯示第幾週。
        ''' </summary>
        < _
        Description("顯示第幾週。"), _
        Category("Appearance"), _
        DefaultValue(False) _
        > _
        Public Property ShowWeekNumbers() As Boolean
            Get
                Return FShowWeekNumbers
            End Get
            Set(ByVal value As Boolean)
                FShowWeekNumbers = value
            End Set
        End Property

        ''' <summary>
        ''' 日期選單樣式。
        ''' </summary>
        < _
        Description("日期選單樣式。"), _
        Category("Appearance"), _
        DefaultValue(GetType(ECalendarStyle), "Blue") _
        > _
        Public Property CalendarStyle() As ECalendarStyle
            Get
                Return FCalendarStyle
            End Get
            Set(ByVal value As ECalendarStyle)
                FCalendarStyle = value
            End Set
        End Property

        ''' <summary>
        ''' 日期選單語系。
        ''' </summary>
        < _
        Description("日期選單語系。"), _
        Category("Appearance"), _
        DefaultValue(GetType(ELang), "Big5") _
        > _
        Public Property Lang() As ELang
            Get
                Return FLang
            End Get
            Set(ByVal value As ELang)
                FLang = value
            End Set
        End Property

        ''' <summary>
        ''' 是否顯示時間。
        ''' </summary>
        < _
        Description("是否顯示時間。"), _
        Category("Appearance"), _
        DefaultValue(False) _
        > _
        Public Property ShowTime() As Boolean
            Get
                Return FShowTime
            End Get
            Set(ByVal value As Boolean)
                FShowTime = value
            End Set
        End Property

        ''' <summary>
        ''' 覆寫 OnLoad 方法。
        ''' </summary>
        Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
            Dim oLink As New System.Web.UI.HtmlControls.HtmlLink()
            Dim sUrl As String
            Dim sScriptPath As String

            '取得用戶端指令碼路徑
            sScriptPath = Me.ScriptPath

            'Header 區段加入 css 樣式表檔案。
            If Me.Page.Header.FindControl("CalendarCss") Is Nothing Then
                sUrl = Me.Page.ResolveUrl(sScriptPath & "css\calendar-" & Me.CalendarStyle.ToString() & ".css")
                oLink.Href = sUrl
                oLink.Attributes("rel") = "stylesheet"
                oLink.Attributes("type") = "text/css"
                oLink.ID = "CalendarCss"
                Me.Page.Header.Controls.Add(oLink)
            End If

            '加入日期控制項所需的 JavaScript 檔案。
            sUrl = sScriptPath & "calendar.js"
            sUrl = Me.Page.ResolveUrl(sUrl)
            Me.Page.ClientScript.RegisterClientScriptInclude("calendar", sUrl)

            sUrl = sScriptPath & "lang/calendar-" & Me.Lang.ToString() & ".js"
            sUrl = Me.Page.ResolveUrl(sUrl)
            Me.Page.ClientScript.RegisterClientScriptInclude("calendar-lang", sUrl)

            sUrl = sScriptPath & "calendar-setup.js"
            sUrl = Me.Page.ResolveUrl(sUrl)
            Me.Page.ClientScript.RegisterClientScriptInclude("calendar-setup", sUrl)

            MyBase.OnLoad(e)
        End Sub

        ''' <summary>
        ''' 建立子控制項。
        ''' </summary>
        Protected Overrides Sub CreateChildControls()
            Me.Controls.Clear()
            FTextBox.ID = "TextBox"
            FTextBox.AutoCompleteType = AutoCompleteType.Disabled
            FButton.ID = "Button"
            FButton.InnerText = "..."
            Me.Controls.Add(FTextBox)
            Me.Controls.Add(FButton)
        End Sub

        ''' <summary>
        ''' 將開頭標記寫入輸出資料流中。
        ''' </summary>
        ''' <param name="writer">資料流。</param>
        Public Overrides Sub RenderBeginTag(ByVal writer As HtmlTextWriter)
            'Me.AddAttributesToRender(writer)
            Dim tagKey As HtmlTextWriterTag = Me.TagKey
            If (tagKey <> HtmlTextWriterTag.Unknown) Then
                writer.RenderBeginTag(tagKey)
            Else
                writer.RenderBeginTag(Me.TagName)
            End If
        End Sub

        ''' <summary>
        ''' 取得用戶端指令碼的參數。
        ''' </summary>
        Private Function GetParamString(ByVal Params As NameValueCollection) As String
            Dim sParam As String
            Dim sName As String
            Dim sValue As String
            Dim N1 As Integer

            sParam = String.Empty
            For N1 = 0 To Params.Count - 1
                sName = Params.Keys(N1)
                sValue = Params.Item(N1)
                If sParam <> String.Empty Then sParam = sParam & ","
                sParam = sParam & sName & ":" & sValue
            Next
            Return sParam
        End Function

        ''' <summary>
        ''' 取得初始化的用戶端指令碼。
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Private Function GetInitScript() As String
            Dim sScript As String
            Dim sParam As String
            Dim oParams As New NameValueCollection()

            oParams.Add("inputField", "'" & FTextBox.ClientID & "'")
            oParams.Add("electric", "false")
            oParams.Add("cache", "true")
            oParams.Add("step", "1")

            If Me.ShowTime Then
                oParams.Add("showsTime", "true")
                oParams.Add("ifFormat", "'%Y/%m/%d %I:%M %p'")
            Else
                oParams.Add("ifFormat", "'%Y/%m/%d'")
            End If

            If Me.FirstDayOfWeek <> UI.WebControls.FirstDayOfWeek.Default Then
                oParams.Add("firstDay", CInt(Me.FirstDayOfWeek).ToString())
            End If

            If Not Me.ShowWeekNumbers Then
                oParams.Add("weekNumbers", "false")
            End If

            sParam = GetParamString(oParams)

            sScript = "Calendar.setup({" & sParam & "});"
            Return sScript
        End Function


        ''' <summary>
        ''' 輸出伺服器控制項內容至資料流。
        ''' </summary>
        ''' <param name="writer">資料流。</param>
        Protected Overrides Sub Render(ByVal writer As System.Web.UI.HtmlTextWriter)
            Dim sScript As String

            '初始化的用戶端指令碼
            sScript = GetInitScript()
            Me.Page.ClientScript.RegisterStartupScript(Me.GetType(), Me.ClientID, sScript, True)

            If Me.ReadOnly Then
                FTextBox.Attributes("readonly") = "readonly"
            End If

            FButton.Attributes("onclick") = FTextBox.ClientID & ".click();"
            If Not Me.Enabled Then
                FButton.Attributes("disabled") = "disabled"
            End If

            FTextBox.CopyBaseAttributes(Me)
            FTextBox.ControlStyle.CopyFrom(Me.ControlStyle)
            Me.Attributes.Clear()
            MyBase.Render(writer)
        End Sub

    End Class
End Namespace